#pragma once
#include <PSX/GPU/gpu_primitives.hpp>
#include <PSX/GPU/gpu.hpp>
#include <PSX/GTE/gte.hpp>

namespace GTETest {
    using namespace JabyEngine;

    struct GTE_Sprite {
        GPU::AreaI16  area;
        GPU::POLY_FT4 display;

        static constexpr GTE_Sprite create(const GPU::POLY_FT4& base) {
            return GTE_Sprite{
                .area    = GPU::AreaI16::create(base.get_rect_pos(), base.get_rect_size()),
                .display = base
            };
        }

        void apply(const GTE::MATRIX& matrix) {
            static const auto apply_to = [](const GTE::MATRIX& matrix, GTE::SVECTOR vector) -> GPU::Vertex {
                GTE::MATRIX  move_back = GTE::MATRIX::identity();
                GTE::SVECTOR output;
                int32_t     flag;

                move_back.trans[0] = -matrix.trans[0];
                move_back.trans[1] = -matrix.trans[1];
                move_back.trans[2] = -matrix.trans[2];

                GTE::comp_matrix(matrix, move_back, move_back);
                return GTE::apply_matrix(move_back, vector, output).to<GPU::Vertex>();
            };

            const auto& area = this->area;
            this->display.vertex0 = apply_to(matrix, GTE::SVECTOR::from(area.get_top_left()));
            this->display.vertex1 = apply_to(matrix, GTE::SVECTOR::from(area.get_top_right()));
            this->display.vertex2 = apply_to(matrix, GTE::SVECTOR::from(area.get_bottom_left()));
            this->display.vertex3 = apply_to(matrix, GTE::SVECTOR::from(area.get_bottom_right()));
        }

        void render() {
            GPU::render(this->display);
        }
    };
}